@@ -4,14 +4,12 @@ from datetime import datetime, timezone
 from functools import wraps
 from typing import Any, Callable, Dict, List, Pattern
 
-from urllib.parse import urlparse
-from moto.core.responses import AWSServiceSpec
-from moto.core.responses import BaseResponse
-from moto.core.responses import xml_to_json_response
+from moto.core.responses import AWSServiceSpec, BaseResponse, xml_to_json_response
 from moto.core.utils import tags_from_query_string
+
 from .exceptions import ValidationException
-from .models import emr_backends, ElasticMapReduceBackend
-from .utils import steps_from_query_string, Unflattener, ReleaseLabel
+from .models import ElasticMapReduceBackend, emr_backends
+from .utils import ReleaseLabel, Unflattener, steps_from_query_string
 
 
 def generate_boto3_response(
@@ -26,7 +24,7 @@ def generate_boto3_response(
     """
 
     def _boto3_request(
-        method: Callable[["ElasticMapReduceResponse"], str]
+        method: Callable[["ElasticMapReduceResponse"], str],
     ) -> Callable[["ElasticMapReduceResponse"], str]:
         @wraps(method)
         def f(self: "ElasticMapReduceResponse") -> str:
@@ -51,7 +49,6 @@ def generate_boto3_response(
 
 
 class ElasticMapReduceResponse(BaseResponse):
-
     # EMR end points are inconsistent in the placement of region name
     # in the URL, so parsing it out needs to be handled differently
     emr_region_regex: List[Pattern[str]] = [
@@ -65,9 +62,8 @@ class ElasticMapReduceResponse(BaseResponse):
         super().__init__(service_name="emr")
 
     def get_region_from_url(self, request: Any, full_url: str) -> str:
-        parsed = urlparse(full_url)
         for regex in ElasticMapReduceResponse.emr_region_regex:
-            match = regex.search(parsed.netloc)
+            match = regex.search(self.parsed_url.netloc)
             if match:
                 return match.group(1)
         return self.default_region
@@ -369,9 +365,9 @@ class ElasticMapReduceResponse(BaseResponse):
             "KerberosAttributes.CrossRealmTrustPrincipalPassword"
         )
         if cross_realm_principal_password:
-            kerberos_attributes[
-                "CrossRealmTrustPrincipalPassword"
-            ] = cross_realm_principal_password
+            kerberos_attributes["CrossRealmTrustPrincipalPassword"] = (
+                cross_realm_principal_password
+            )
 
         ad_domain_join_user = self._get_param("KerberosAttributes.ADDomainJoinUser")
         if ad_domain_join_user:
@@ -405,7 +401,7 @@ class ElasticMapReduceResponse(BaseResponse):
                 cluster.id, instance_groups
             )
             for i in range(0, len(instance_group_result)):
-                self.backend.add_instances(
+                self.backend.run_instances(
                     cluster.id, instance_groups[i], instance_group_result[i]
                 )
 
@@ -475,9 +471,9 @@ class ElasticMapReduceResponse(BaseResponse):
                     )
                 if vol_type in ebs_configuration:
                     instance_group.pop(vol_type)
-                    ebs_block[volume_specification][
-                        volume_type
-                    ] = ebs_configuration.pop(vol_type)
+                    ebs_block[volume_specification][volume_type] = (
+                        ebs_configuration.pop(vol_type)
+                    )
 
                 per_instance = f"{key}._{volumes_per_instance}"
                 if per_instance in ebs_configuration:
@@ -539,6 +535,45 @@ class ElasticMapReduceResponse(BaseResponse):
         template = self.response_template(REMOVE_AUTO_SCALING_POLICY)
         return template.render()
 
+    @generate_boto3_response("GetBlockPublicAccessConfiguration")
+    def get_block_public_access_configuration(self) -> str:
+        configuration = self.backend.get_block_public_access_configuration()
+        config = configuration.get("block_public_access_configuration") or {}
+        metadata = configuration.get("block_public_access_configuration_metadata") or {}
+        template = self.response_template(
+            GET_BLOCK_PUBLIC_ACCESS_CONFIGURATION_TEMPLATE
+        )
+        return template.render(
+            block_public_security_group_rules=config.get(
+                "block_public_security_group_rules"
+            ),
+            permitted_public_security_group_rule_ranges=config.get(
+                "permitted_public_security_group_rule_ranges"
+            ),
+            creation_date_time=metadata.get("creation_date_time"),
+            created_by_arn=metadata.get("created_by_arn"),
+        )
+
+    @generate_boto3_response("PutBlockPublicAccessConfiguration")
+    def put_block_public_access_configuration(self) -> str:
+        params = self._get_params()
+        block_public_access_configuration = (
+            params.get("BlockPublicAccessConfiguration") or {}
+        )
+        self.backend.put_block_public_access_configuration(
+            block_public_security_group_rules=block_public_access_configuration.get(
+                "BlockPublicSecurityGroupRules"
+            )
+            or True,
+            rule_ranges=block_public_access_configuration.get(
+                "PermittedPublicSecurityGroupRuleRanges"
+            ),
+        )
+        template = self.response_template(
+            PUT_BLOCK_PUBLIC_ACCESS_CONFIGURATION_TEMPLATE
+        )
+        return template.render()
+
 
 ADD_INSTANCE_GROUPS_TEMPLATE = """<AddInstanceGroupsResponse xmlns="http://elasticmapreduce.amazonaws.com/doc/2009-03-31">
   <AddInstanceGroupsResult>
@@ -761,7 +796,7 @@ DESCRIBE_JOB_FLOWS_TEMPLATE = """<DescribeJobFlowsResponse xmlns="http://elastic
               <InstanceRequestCount>{{ instance_group.num_instances }}</InstanceRequestCount>
               <InstanceRole>{{ instance_group.role }}</InstanceRole>
               <InstanceRunningCount>{{ instance_group.num_instances }}</InstanceRunningCount>
-              <InstanceType>{{ instance_group.type }}</InstanceType>
+              <InstanceType>{{ instance_group.instance_type }}</InstanceType>
               <LastStateChangeReason/>
               <Market>{{ instance_group.market }}</Market>
               <Name>{{ instance_group.name }}</Name>
@@ -1087,7 +1122,7 @@ LIST_INSTANCE_GROUPS_TEMPLATE = """<ListInstanceGroupsResponse xmlns="http://ela
         {% endif %}
         <Id>{{ instance_group.id }}</Id>
         <InstanceGroupType>{{ instance_group.role }}</InstanceGroupType>
-        <InstanceType>{{ instance_group.type }}</InstanceType>
+        <InstanceType>{{ instance_group.instance_type }}</InstanceType>
         <Market>{{ instance_group.market }}</Market>
         <Name>{{ instance_group.name }}</Name>
         <RequestedInstanceCount>{{ instance_group.num_instances }}</RequestedInstanceCount>
@@ -1422,3 +1457,36 @@ DELETE_SECURITY_CONFIGURATION_TEMPLATE = """<DeleteSecurityConfigurationResponse
     <RequestId>2690d7eb-ed86-11dd-9877-6fad448a8419</RequestId>
   </ResponseMetadata>
 </DeleteSecurityConfigurationResponse>"""
+
+PUT_BLOCK_PUBLIC_ACCESS_CONFIGURATION_TEMPLATE = """<PutBlockPublicAccessConfigurationResponse xmlns="http://elasticmapreduce.amazonaws.com/doc/2009-03-31">
+  <ResponseMetadata>
+    <RequestId>2690d7eb-ed86-11dd-9877-6fad448a8419</RequestId>
+  </ResponseMetadata>
+</PutBlockPublicAccessConfigurationResponse>"""
+
+GET_BLOCK_PUBLIC_ACCESS_CONFIGURATION_TEMPLATE = """
+  <GetBlockPublicAccessConfigurationResponse xmlns="http://elasticmapreduce.amazonaws.com/doc/2009-03-31">
+    <GetBlockPublicAccessConfigurationResult>
+      <BlockPublicAccessConfiguration>
+        <BlockPublicSecurityGroupRules>
+          {{block_public_security_group_rules}}
+        </BlockPublicSecurityGroupRules>
+        <PermittedPublicSecurityGroupRuleRanges>
+          {% for rule_range in permitted_public_security_group_rule_ranges %}
+            <member>
+              <MinRange>{{rule_range['min_range']}}</MinRange>
+              <MaxRange>{{rule_range['max_range']}}</MaxRange>      
+            </member>
+          {% endfor %}
+        </PermittedPublicSecurityGroupRuleRanges>
+      </BlockPublicAccessConfiguration>
+      <BlockPublicAccessConfigurationMetadata>
+        <CreationDateTime>{{creation_date_time}}</CreationDateTime>
+        <CreatedByArn>{{created_by_arn}}</CreatedByArn>
+      </BlockPublicAccessConfigurationMetadata>
+    </GetBlockPublicAccessConfigurationResult>
+    <ResponseMetadata>
+      <RequestId>2690d7eb-ed86-11dd-9877-6fad448a8419</RequestId>
+    </ResponseMetadata>
+  </GetBlockPublicAccessConfigurationResponse>
+"""
